@@ -28,39 +28,66 @@
 #' user will be prompted through a menu to individually refit observations
 #' @import data.table
 #' @export
-bdotsRefit <- function(bdObj, fitCode = 1L,
-                       quickRefit = FALSE, numRefits = 2L,
-                       paramDT = NULL, ...) {
+bdotsRefit <- function(bdObj, fitCode = 1L, quickRefit = FALSE,
+                       numRefits = 2L, paramDT = NULL, ...) {
 
   if (is.null(fitCode)) fitCode <- 1L
 
-
   if (is.null(attr(bdObj, "X")$X)) {
     stop("Dataset must be provided")
   }
 
   ## These uniquely identify each fit
   bdCall <- attr(bdObj, "call")
-  nn <- getIdentifierCols(bdObj) #c(eval(bdCall[['subject']]), eval(bdCall[['group']]))
+  nn <- getIdentifierCols(bdObj)
 
   if (!is.null(paramDT) & inherits(paramDT, what = "data.frame")) {
     res <- mergecoef(bdObj, paramDT)
     idx <- res[['idx']]
     new_bd <- res[['new_bd']]
   } else {
+
     fitcode <- fitCode
-    idx <- which(bdObj$fitCode >= fitcode)
+
+    ## Check if any refit has already occurred, make subset index
+    HAS_PRIOR_REFIT <- attr(bdObj, "refit_idx") # old refit
+    if (!is.null(HAS_PRIOR_REFIT)) {
+      ..nn <- NULL
+      bd_identifiers <- do.call(paste, bdObj[, ..nn]) # all subjects
+      NEEDS_REFIT_IDX <- !(bd_identifiers %in% HAS_PRIOR_REFIT)
+      idx <- which(bdObj$fitCode >= fitcode & NEEDS_REFIT_IDX)
+    } else {
+      idx <- which(bdObj$fitCode >= fitCode)
+    }
+
+    ## Handle case where no refits
     if (length(idx) == 0L) {
       message(paste0("All observations fitCode greater than ",
                      fitcode, ". Nothing to refit :)"))
       return(bdObj)
     }
+
+    ## Split by refit index above
     bdObj2 <- split(bdObj[idx, ], by = nn)
+
     if (quickRefit) {
       ## Oddly, I get errors running parLapply not lapply. Will investigate
       new_bd <- lapply(bdObj2, bdQuickRefit, numRefits)
     } else {
-      new_bd <- lapply(bdObj2, bdUpdate, numRefits)
+      new_bd <- vector("list", length = length(bdObj2))
+      BREAK <- FALSE # for ending sequence mid fits
+      for (i in seq_along(new_bd)) {
+        new_bd[[i]] <- bdUpdate(bdObj2[[i]], numRefits)
+        if (BREAK) break # break loop if done fitting
+      }
+      if (BREAK) {
+        REFIT_IDX <- idx[seq_len(i-1)]
+        new_bd <- new_bd[seq_len(i-1)] # otherwise, new_bd will have null entries which are considered deleted
+      } else {
+        REFIT_IDX <- idx
+      }
+      HAS_REFIT <- do.call(paste, bdObj[REFIT_IDX, ..nn])
+      HAS_REFIT <- c(HAS_REFIT, HAS_PRIOR_REFIT)
     }
   }
 
@@ -118,10 +145,11 @@ bdotsRefit <- function(bdObj, fitCode = 1L,
         bdObj <- bdObj[-rmv_pairs, ]
     }
   }
+  if (exists("HAS_REFIT")) attr(bdObj, "refit_idx") <- HAS_REFIT
   bdObj
 }
 
-
+## Performs number of refits until improve fitCode/R2
 bdQuickRefit <- function(bdo, numRefits) {
 
   if (bdo$fitCode != 6L) {
@@ -155,7 +183,8 @@ bdQuickRefit <- function(bdo, numRefits) {
 
 
 ## Given a single bdotsFit observation, computes a refit
-bdRefitter <- function(bdo, numRefits = 0L, rho = NULL, params = NULL, ...) {
+bdRefitter <- function(bdo, numRefits = 0L, rho = NULL,
+                       params = NULL, getCovOnly = NULL, ...) {
   if (nrow(bdo) != 1L) stop("bdRefitter can only take a single observation")
   bdCall <- attr(bdo, "call")
   nn <- getIdentifierCols(bdo) #c(eval(bdCall[['subject']]), eval(bdCall[['group']])) # this is split vars!
@@ -168,7 +197,7 @@ bdRefitter <- function(bdo, numRefits = 0L, rho = NULL, params = NULL, ...) {
 
   new_bdo <- bdotsFitter(dat = x, curveType = crvFun, rho = rho,
                          params = params, splitVars = nn, datVarNames = bdCall,
-                         numRefits = numRefits)
+                         numRefits = numRefits, getCovOnly = getCovOnly)
   prob <- tryCatch(attributes(new_bdo) <- attributes(bdo), error = function(e) 2)
   if (is.numeric(prob)) stop("issue in bdrefitter, likely with ncols")
   new_bdo
@@ -204,12 +233,20 @@ bdUpdate <- function(bdo, numRefits) {
                      "3) Adjust starting parameters manually\n",
                      "4) Remove AR1 assumption\n",
                      "5) See original fit metrics\n",
-                     "6) Delete subject")
+                     "6) Delete subject\n",
+                     "99) Save and exit refitter")
     cat(rf_msg)
     resp <- NA
-    while (!(resp %in% 1:6)) {
+    while (!(resp %in% c(1:6, 99))) {
       resp <- readline("Choose (1-6): ")
     }
+
+    if (resp == 99) {
+      assign("BREAK", TRUE, pos = parent.frame())
+      accept <- TRUE
+      break
+    }
+
     if (resp == 1) {
       accept <- TRUE
       break
@@ -238,6 +275,22 @@ bdUpdate <- function(bdo, numRefits) {
         }
       }
       class(newPars) <- "numeric"
+
+      ## modify AR1 status
+      cat("Current AR1 Assumption:\n")
+      print(bdo$AR1)
+      #tmpval <- NA
+      while (TRUE) {
+        ar_val <- readline("Use AR1 assumption? (y/n): ")
+        if (ar_val %in% c("y", "n", "")) {
+          has_ar <- bdo$fitCode < 3
+          rho <- ifelse(ar_val == "n", 0, rho*has_ar)
+          break
+        } else {
+          cat("Please enter 'y' or 'n'\n")
+        }
+      }
+
     } else if (resp == 4) {
       rho <- 0
       newPars <- oldPars
@@ -261,7 +314,11 @@ bdUpdate <- function(bdo, numRefits) {
         next # reset while loop
     }
 
-    new_bdo <- bdRefitter(bdo, rho, numRefits, params = newPars)
+    new_bdo <- bdRefitter(bdo, numRefits, rho, params = newPars)
+
+    ## Reset rho in case it was turned to zero
+    rho <- attr(bdo, "rho")
+
     both_bdo <- structure(.Data = list(bdo, new_bdo),
                           class = "bdObjList")
 
@@ -343,12 +400,21 @@ bdUpdate_NULL <- function(bdo, numRefits) {
     if (rounds == 1L) {
       rf_msg <- paste0("\nActions:\n",
                        "1) Adjust starting parameters manually\n",
-                       "2) Delete subject")
+                       "2) Delete subject\n",
+                       "99) Save and exit refitter")
       cat(rf_msg)
       resp <- NA
-      while (!(resp %in% 1:2)) {
+      while (!(resp %in% c(1:2, 99))) {
         resp <- readline("Choose (1-2): ")
       }
+
+      if (resp == 99) {
+        assign("BREAK", TRUE, pos = parent.frame())
+        accept <- TRUE
+        break
+      }
+
+
       ## resp = 1:4 changes, so need to map 2 here to delete subject (4)
       # in case they change their mind (selecting 'n' for delete), it needs to come
       # back to this original menu prompt. Consequently, update 'rounds' only if reps == 1
@@ -364,14 +430,21 @@ bdUpdate_NULL <- function(bdo, numRefits) {
                        "1) Adjust starting parameters manually\n",
                        "2) Print previous parameter attempt\n",
                        "3) Update previous parameter attempt\n",
-                       "4) Delete subject")
+                       "4) Delete subject\n",
+                       "99) Save and exit refitter")
       cat(rf_msg)
       resp <- NA
-      while (!(resp %in% 1:4)) {
+      while (!(resp %in% c(1:4, 99))) {
         resp <- readline("Choose (1-4): ")
       }
     }
 
+    if (resp == 99) {
+      assign("BREAK", TRUE, pos = parent.frame())
+      accept <- TRUE
+      break
+    }
+
     if (resp == 1) {
       parnames <- attributes(attr(bdo, "formula"))[['parnames']]
       newPars <- setNames(rep(NA, length(parnames)), parnames)
@@ -386,7 +459,18 @@ bdUpdate_NULL <- function(bdo, numRefits) {
         }
       }
 
-      new_bdo <- bdRefitter(bdo, rho = 0.9, numRefits, params = newPars)
+      ## determine AR status to be used
+      while (TRUE) {
+        ar_val <- readline("Use AR1 assumption? (y/n): ")
+        if (ar_val %in% c("y", "n", "")) {
+          rho <- ifelse(ar_val == "n", 0, 0.9)
+          break
+        } else {
+          cat("Please enter 'y' or 'n'\n")
+        }
+      }
+
+      new_bdo <- bdRefitter(bdo, numRefits, rho, params = newPars)
 
       if (new_bdo$fitCode == 6) {
         cat("Fit unsuccessful. Plotting curve from your input parameters in red.\n",
@@ -427,7 +511,18 @@ bdUpdate_NULL <- function(bdo, numRefits) {
             warning("Invalid entry, please enter a numeric value")
         }
       }
-      new_bdo <- bdRefitter(bdo, rho = 0.9, numRefits, params = newPars)
+      ## adjust ar1 status
+      while (TRUE) {
+        ar_val <- readline("Use AR1 assumption? (y/n): ")
+        if (ar_val %in% c("y", "n", "")) {
+          rho <- ifelse(ar_val == "n", 0, 0.9)
+          break
+        } else {
+          cat("Please enter 'y' or 'n'\n")
+        }
+      }
+
+      new_bdo <- bdRefitter(bdo, numRefits, rho, params = newPars)
       if (new_bdo$fitCode == 6) {
         cat("Fit unsuccessful. Plotting curve from your input parameters in red.\n",
             "Use this to adjust parameter estimates accordingly\n")
